/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
const CLASS_ID = Components.ID("{71B45B67-5AC7-4aaf-8043-A3DD45C67636}");
const CLASS_NAME = "Forecastfox Ping Component";
const CONTRACT_ID = "@ensolis.com/forecastfox/ping;1";
const ffIPing = Components.interfaces.ffIPing;

/******************************************************************************
 * ffPing Component
 ******************************************************************************/
function ffPing()
{
  this.wrappedJSObject = this;
};

ffPing.prototype = {
  _URL: "http://forecastfox.ensolis.com/logs/log.php",
  _branch: null,
  _ds: null,
  _app: null,
  get url() { return this._URL; },
  
  start: function()
  {
    //register as an observer of the em datasource
    try {
      var em = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
      this._ds = em.datasource.QueryInterface(Components.interfaces.nsIRDFDataSource);
      this._ds.AddObserver(this);
    } catch(e) {};

    //get the app.id value
    try {
      this._app = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);    
    } catch(e) {};
          
    //get pref branch
    var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);        
    this._branch = pbs.getBranch("forecastfox.");
    var last = this._branch.getCharPref("pinged");
    var migrated = this._branch.getCharPref("migrated");
          
    //use migrated value if last is blank
    if (last == "" && migrated != "")
      last = migrated;
    
    switch (last) {
      case "0.7.8":
        break;    
      case "":
      case "*":
        this.open(ffIPing.OPEN_INSTALL, null, "0.7.8");
        break;
      default:       
        this.open(ffIPing.OPEN_UPGRADE, last, "0.7.8");
        break;
    };
    
    //set pref and lock so user can't update
    this._branch.setCharPref("pinged", "0.7.8");
    this._branch.lockPref("pinged");
  },
  
  stop: function()
  {
    //unregister as an observer of the em datasource  
    if (this._ds)
      this._ds.RemoveObserver(this);
    
    //unlock preference  
    this._branch.unlockPref("pinged");
    
    //remove variables
    this._branch = null;
    this._app = null;
    this._ds = null;
  },
  
  open: function(aType, aFromVer, aToVer)
  { 
    //create query url
    var query = this._URL + "?type=" + aType;
    switch (aType) {
      case ffIPing.OPEN_UNINSTALL:
        query = query + "&from=" + aFromVer;
        break;
      case ffIPing.OPEN_UPGRADE:
        query = query + "&from=" + aFromVer + "&to=" + aToVer;
        break;
      case ffIPing.OPEN_INSTALL:
        query = query + "&to=" + aToVer;
        break;
    };
    
    //add app id
    var guid;
    if (this._app)
      guid = this._app.ID;
    else {
      var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);        
      var appBranch = pbs.getBranch(null);
      try {
        guid = appBranch.getCharPref("app.id");
      } catch(e) {
        guid = "{86c18b42-e466-45a9-ae7a-9b95ba6f5640}";
      };
    }
    query = query + "&guid=" + guid; 
            
    //send ping request
    var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var channel = ios.newChannel(query, null, null);   
    channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE;
    channel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
    channel.requestMethod = "GET";
    channel.open();
  },

  _doUninstall: function(aDS, aSource, aProperty, aTarget)
  {
    //not us
    if (aSource.ValueUTF8 != "urn:mozilla:extension:" + "{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}")
      return;
      
    //not toBeUninstalled
    if (aProperty.ValueUTF8 != "http://www.mozilla.org/2004/em-rdf#toBeUninstalled")   
      return;
    
    //do ping
    this.open(ffIPing.OPEN_UNINSTALL, "0.7.8", null); 
    
    //set special pinged value
    this._branch.unlockPref("pinged");
    this._branch.setCharPref("pinged", "*");
    this._branch.lockPref("pinged");
  },
  
  ///////////////////////////
  // nsIRDFObserver
  onAssert: function(aDS, aSource, aProperty, aTarget) { this._doUninstall(aDS, aSource, aProperty, aTarget); },  
  onUnassert: function(aDS, aSource, aProperty, aTarget) {},
  onChange: function(aDS, aSource, aProperty, aOldTarget, aNewTarget) {},  
  onMove: function(aDS, aOldSource, aNewSource, aProperty, aTarget) {},
  onBeginUpdateBatch: function(aDS) {},
  onEndUpdateBatch: function(aDS) {},
        
  ///////////////////////////
  // nsISupports
  QueryInterface: function (aIID)
  {
    if (!aIID.equals(Components.interfaces.ffIPing) &&
        !aIID.equals(Components.interfaces.nsISupports) &&
        !aIID.equals(Components.interfaces.nsIRDFObserver))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

/******************************************************************************
 * XPCOM Functions for construction and registration
 ******************************************************************************/
var gModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    if (this._firstTime) {
      this._firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    };
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return gFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

var gFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new ffPing()).QueryInterface(aIID);
  }
};

function NSGetModule(aCompMgr, aFileSpec) { return gModule; }