/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gSearch = null;

function searchLoad()
{
  gSearch = new SearchModule();
  gSearch.start();
};

function searchUnload()
{
  gSearch.stop();
  gSearch = null;
};

function SearchModule()
{
  this.wrappedJSObject = this;
  return this;
};

SearchModule.prototype = {
  _manager: null,
  _request: null,
  _bundle: null,
  _textbox: null,
  _results: null,
  _running: false,
  _baseURL: "http://forecastfox.accuweather.com/adcbin/forecastfox/locate_city.asp?location=",
       
  start: function()
  {
    this._manager  = Components.classes["@ensolis.com/forecastfox/manager;1"].getService(Components.interfaces.ffIManager); 
    this._bundle  = document.getElementById("ff-bundle-search");
    this._textbox = document.getElementById("ff-text-search");
    this._textbox.focus();
    this._textbox.select();
    this._results = document.getElementById("ff-items-results");
  },
  
  stop: function()
  {
    this._manager = null;
    this._running = null;
    this._request = null;
    this._bundle = null;
    this._baseURL = null;
    this._textbox = null;
    this._results = null;
  },
  
  run: function()
  {
    //remove old search
    this._clear();
    
    //do nothing if string not in textbox or request running
    if (!this._textbox.value || this._running)
      return;  
        
    //set running
    this._running = true;

    //set label
    this._setLabel("ff.search.loading", null);

    //set url
    var URL = encodeURI(this._baseURL + this._textbox.value);

    //setup request and send
    var module = this;
    this._request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);
    this._request.open("GET", URL, true);
    this._request.overrideMimeType("text/xml");
    this._request.onload = function() { module._onSearchLoad(this.responseXML, this.status) };
    this._request.onerror = function() { module._onSearchError() };
    this._request.send(null);          
  },
  
  _clear: function()
  {
    while (this._results.hasChildNodes())
      this._results.removeChild(this._results.lastChild);    
  },
  
  _setLabel: function(aText, aNode)
  {
    var treeitem, treerow, treecell1, treecell2, treecell3;  

    treeitem = document.createElement("treeitem");
    treerow = document.createElement("treerow");
    treecell1 = document.createElement("treecell");
    treecell2 = document.createElement("treecell");  
    treecell3 = document.createElement("treecell");
          
    if (aText) {
      treeitem.setAttribute("id", "na");  
      treecell1.setAttribute("label", this._bundle.getString(aText));
      treecell2.setAttribute("label", "");
      treecell3.setAttribute("label", "");  
    } else {
      treeitem.setAttribute("id", aNode.getAttribute("location"));  
      treecell1.setAttribute("label", aNode.getAttribute("city"));
      treecell2.setAttribute("label", aNode.getAttribute("state"));
      treecell3.setAttribute("label", aNode.getAttribute("location"));     
    };
    
    treerow.appendChild(treecell1);
    treerow.appendChild(treecell2);  
    treerow.appendChild(treecell3);
    treeitem.appendChild(treerow);
    this._results.appendChild(treeitem);
  },
  
  _onSearchLoad: function(aResponse, aStatus)
  { 
    //set running to false
    this._running = false;
  
    //clear old results
    this._clear();
  
    //response error
    if (!aResponse || aStatus > 200) {
      this._setLabel("ff.search.error", null);
      return;
    };

    //no locations
    var locs = aResponse.getElementsByTagName("location");
    if (locs.length == 0) {
      this._setLabel("ff.search.location", null);
      return;
    };

    //populate tree
    for (var i=0; i < locs.length; i++)
      this._setLabel(null, locs[i]);
  },
  
  _onSearchError: function()
  {
    //set running to false
    this._running = false;
  
    //clear old results
    this._clear();
  

    //set error in tree
    this._setLabel("ff.search.error", null);  
  },
  
  accept: function(aClose)
  {
    var tree = document.getElementById("ff-tree-results");
  
    //if nothing selected return
    if (tree.currentIndex == -1) {
      if (aClose) {
        window.close();
        return;
      } else
      return;
    };
  
    //if no location return
    var current = tree.view.getItemAtIndex(tree.currentIndex).getAttribute("id");
    if (current == "na") {
      if (aClose) {
        window.close();
        return;
      } else
      return;
    };

    //set location value
    var locid = window.opener.document.getElementById("ff-text-code");
    locid.value = current;
  
    //update apply button
    window.opener.updateButtons();
    window.close();  
  },
  
  openLink: function(aEvent, aPartner)
  {
    var url = aEvent.currentTarget.getAttribute("url");
    if (aPartner)
      url = url + "?partner=" + this._manager.partner;

	var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService); 
    rootBranch = prefService.getBranch("");
    //XXX need app specific way that doesn't use app.id pref
    try {
      rootBranch.getCharPref("app.id");
      window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", url, null, null);
    } catch (e) {
      window.openDialog("chrome://navigator/content/navigator.xul", "_blank", "chrome,all,dialog=no", url, null, null);
    }
  }
}